package ru.esc.btqckcmd.ui;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;

import ru.esc.btqckcmd.service.IRemoteService;
import ru.esc.btqckcmd.service.RemoteService;
import ru.esc.btqckcmd.service.RemoteService.RemoteServiceBinder;
import ru.esc.btqckcmd.utils.MLog;

import android.app.Activity;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.bluetooth.BluetoothAdapter;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.graphics.Point;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.view.Display;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.View.OnClickListener;
import android.view.ViewGroup.LayoutParams;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import ru.esc.btqckcmd.R;

public class MainActivity extends Activity implements OnClickListener,
		IRemoteService {

    private static final int DISPLAY_MAX_HEIGHT = 320;
    private static final int DISPLAY_MAX_WIDTH = 240;

	/**
	 * Request code used when {@link #startActivityForResult(Intent, int)}
	 * called to show dialog for turning Bluetooth on.
	 */
	private static final int REQUEST_ENABLE_BT = 101;

	/**
	 * Msg codes used for {@link #mHandler} object.
	 */
	private static final int MSG_ADD_STATUS = 10;
	private static final int MSG_SCAN_STARTED = 11;
	private static final int MSG_CONNECTED = 12;
	private static final int MSG_NOT_FOUND = 13;
	private static final int MSG_NOT_CONNECTED = 14;
	private static final int MSG_DISCONNECTED = 15;
	private static final int MSG_SERVICE_BOUND = 16;

	/**
	 * Dialog id for {@link #mPdConnect} {@link ProgressDialog}
	 */
	private static final int DIALOG_CONNECT = 1;

	/**
	 * {@link ProgressDialog} shown while connecting to Bluetooth device.
	 */
	private ProgressDialog mPdConnect = null;
	/**
	 * Indicates if {@link ProgressDialog} is active.
	 */
	private boolean mIsDialogActive = false;

	/**
	 * Indicates if {@link RemoteService} is bound.
	 */
	private boolean mBound = false;

	/**
	 * {@link RemoteService} object.
	 */
	private RemoteService mService = null;

	/**
	 * {@link BluetoothAdapter} object. Used to check if bluetooth is turned on.
	 */
	private BluetoothAdapter mBluetoothAdapter = null;

	/**
	 * Views in current Activity.
	 */
	private Button mBtConnect;
	private TextView mTvStatus;

	public class SimpleButton extends Button {

        private final int mCommandNumber;
        private int mButtonWidth;
        private int mButtonHeight;

	    public SimpleButton(Context context, int command) {
	        super(context);

	        Display display = getWindowManager().getDefaultDisplay();
	        Point size = new Point();
	        overrideGetSize(display, size);
	        int width = size.x;
	        int height = size.y;
            mButtonWidth = width / 4;
            mButtonHeight = height / 20;

	        mCommandNumber = command;
	        setWidth(mButtonWidth);
	        setHeight(mButtonHeight);
	        setText(Integer.toString(command));
	        setOnClickListener(new android.view.View.OnClickListener() {
                
                @Override
                public void onClick(View arg0) {
                    if (mBound) {
                        mService.sendCommand(mCommandNumber);
                    }
                }

	        });
	    }

        public SimpleButton(Context context, int command, String btnLabel) {
            super(context);
            mCommandNumber = command;
            setLayoutParams(new ViewGroup.LayoutParams(ViewGroup.LayoutParams.FILL_PARENT, ViewGroup.LayoutParams.FILL_PARENT));
            setText(btnLabel);
            setOnClickListener(new android.view.View.OnClickListener() {
                
                @Override
                public void onClick(View arg0) {
                    if (mBound) {
                        mService.sendCommand(mCommandNumber);
                    }
                }

            });
        }

	}

	public class SimpleButtonManager {
	    private int mCurrentButtonCount;
        private LinearLayout mButtonsLayout;
        private LinearLayout mLatestHorLayout;

        public SimpleButtonManager(LinearLayout buttonsLayout) {
            mCurrentButtonCount = 0;
            mButtonsLayout = buttonsLayout;
        }

        int addSmallBtn(SimpleButton btn) {
            if (mCurrentButtonCount % 3 == 0) {
                LinearLayout horizontalLayout = new LinearLayout(MainActivity.this);
                horizontalLayout.setOrientation(LinearLayout.HORIZONTAL);
                horizontalLayout.setLayoutParams(new ViewGroup.LayoutParams(
                        ViewGroup.LayoutParams.FILL_PARENT, 
                        ViewGroup.LayoutParams.WRAP_CONTENT));
                mLatestHorLayout = horizontalLayout;
            }
            mLatestHorLayout.addView(btn);
            if (mCurrentButtonCount % 3 == 0) {
                mButtonsLayout.addView(mLatestHorLayout);
            }
            mCurrentButtonCount ++;
            return 0;
        }

        int addLargeBtn(SimpleButton btn) {
            LinearLayout horizontalLayout = new LinearLayout(MainActivity.this);
            horizontalLayout.setOrientation(LinearLayout.HORIZONTAL);
            horizontalLayout.setLayoutParams(new ViewGroup.LayoutParams(
                    ViewGroup.LayoutParams.FILL_PARENT, 
                    ViewGroup.LayoutParams.WRAP_CONTENT));
            horizontalLayout.addView(btn);
            mButtonsLayout.addView(horizontalLayout);
            return 0;
        }
	}

	void overrideGetSize(Display display, Point outSize) {
	    try {
	      // test for new method to trigger exception
	      Class pointClass = Class.forName("android.graphics.Point");
	      Method newGetSize = Display.class.getMethod("getSize", new Class[]{ pointClass });

	      // no exception, so new method is available, just use it
	      newGetSize.invoke(display, outSize);
	    } catch(NoSuchMethodException ex) {
	      // new method is not available, use the old ones
	      outSize.x = display.getWidth();
	      outSize.y = display.getHeight();
	    } catch (ClassNotFoundException e) {
            e.printStackTrace();
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        }
	}

	/**
	 * Initializes views and some other objects of Activity.
	 * 
	 * @see android.app.Activity#onCreate(android.os.Bundle)
	 */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main_layout);

		mBtConnect = (Button) findViewById(R.id.ml_bt_connect);
		mBtConnect.setOnClickListener(MainActivity.this);

        // init BluetoothAdapter object
        mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();

		LinearLayout buttonsLayout = (LinearLayout) findViewById(R.id.buttons_layout);
		
		SimpleButtonManager btnMgr = new SimpleButtonManager(buttonsLayout);
        btnMgr.addSmallBtn(new SimpleButton(this, 20));
        btnMgr.addSmallBtn(new SimpleButton(this, 30));
        btnMgr.addSmallBtn(new SimpleButton(this, 40));
        btnMgr.addSmallBtn(new SimpleButton(this, 50));
        btnMgr.addSmallBtn(new SimpleButton(this, 60));
        btnMgr.addSmallBtn(new SimpleButton(this, 70));
        btnMgr.addSmallBtn(new SimpleButton(this, 80));
        btnMgr.addSmallBtn(new SimpleButton(this, 90));
        btnMgr.addLargeBtn(new SimpleButton(this, 128, "Some command to do smth"));
        btnMgr.addSmallBtn(new SimpleButton(this, 33));
        btnMgr.addSmallBtn(new SimpleButton(this, 88));
        btnMgr.addSmallBtn(new SimpleButton(this, 99));
        btnMgr.addSmallBtn(new SimpleButton(this, 144));
	}

	/**
	 * Start and bind service when activity started.
	 * 
	 * @see android.app.Activity#onStart()
	 */
	@Override
	protected void onStart() {
		super.onStart();
		startService(new Intent(MainActivity.this, RemoteService.class));
		MLog.i("MainActivity start; bound = " + mBound);
		if (!mBound) {
			bindService();
		}
	}

	/**
	 * Unbind service when activity stopped.
	 * 
	 * @see android.app.Activity#onStop()
	 */
	@Override
	protected void onStop() {
		super.onStop();
		MLog.i("MainActivity stop; bound = " + mBound);
		if (mBound) {
			unbindService();
		}
	}

	/**
	 * Binds service. Async. When service is bound
	 * {@link ServiceConnection#onServiceConnected(ComponentName, IBinder)}
	 * called in {@link #mServiceConnection} object.
	 */
	private void bindService() {
		if (!mBound) {
			// addStatusMsg("Bind service");
			// Bind to LocalService
			Intent intent = new Intent(MainActivity.this, RemoteService.class);
			bindService(intent, mServiceConnection, Context.BIND_AUTO_CREATE);
		}
	}

	/**
	 * Unbinds service.
	 */
	public void unbindService() {
		if (mBound) {
			addStatusMsg("Unbind service");
			unbindService(mServiceConnection);
			mBound = false;
		}
	}

	/**
	 * Checks if bluetooth is turned on. If not, ask user to turn it on.
	 * 
	 * @return {@code true} if Bluetooth is currently turned on.
	 */
	private boolean checkBluetoothState() {
		if (mBluetoothAdapter.isEnabled())
			return true;
		else {
			Intent enableBtIntent = new Intent(
					BluetoothAdapter.ACTION_REQUEST_ENABLE);
			startActivityForResult(enableBtIntent, REQUEST_ENABLE_BT);
			return false;
		}
	}

	/**
	 * @see android.view.View.OnClickListener#onClick(android.view.View)
	 */
	public void onClick(View v) {
		switch (v.getId()) {
		case R.id.ml_bt_connect:
		    if (mBtConnect.getText().equals(getString(R.string.connect))) {
	            if (checkBluetoothState()) {
	                if (mBound) {
	                    if (!mService.isConnected()) {
	                        if (!mService.isConnecting()) {
	                            mService.startDeviceDiscover();
	                        } else {
	                            showConnectingDialog();
	                        }
	                    } else {
	                        addStatusMsg("Bluetooth device is already connected.");
	                    }
	                } else {
	                    bindService();
	                }
	            }
		    } else {
		        if (mBound) {
		            unbindService();
		        }
		    }
			break;
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.main, menu);
	    return super.onCreateOptionsMenu(menu);
	}

	@Override
	public boolean onMenuItemSelected(int featureId, MenuItem item) {
	    switch (item.getItemId()) {
            case R.id.action_settings:
                startActivity(new Intent(getApplicationContext(), GeneralPreferences.class));
            default:
                return super.onMenuItemSelected(featureId, item);
	    }
	}

	/**
	 * @see android.app.Activity#onCreateDialog(int)
	 */
	@Override
	protected Dialog onCreateDialog(int id) {
		switch (id) {
		case DIALOG_CONNECT:
			mPdConnect = new ProgressDialog(MainActivity.this);
			mPdConnect.setCancelable(false);
			mPdConnect.setMessage("Connecting...");
			return mPdConnect;

		default:
			return super.onCreateDialog(id);
		}
	}

	/**
	 * Shows progress dialog when connecting to remote device.
	 */
	private void showConnectingDialog() {
		if (!mIsDialogActive) {
			showDialog(DIALOG_CONNECT);
			mIsDialogActive = true;
		}
	}

	/**
	 * Dismiss progress dialog when connecting to remote device.
	 */
	private void dismissConnectingDialog() {
		if (mIsDialogActive) {
			dismissDialog(DIALOG_CONNECT);
			mIsDialogActive = false;
		}
	}

    /**
	 * @see android.app.Activity#onActivityResult(int, int,
	 *      android.content.Intent)
	 */
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		if (requestCode == REQUEST_ENABLE_BT) {
			if (resultCode == RESULT_OK) {
				MLog.i("BT turned ON");
				addStatusMsg("BT turned ON");
				mService.startDeviceDiscover();
			} else {
				MLog.i("BT is OFF");
				addStatusMsg("BT is OFF");
			}
		}
	}

	/**
	 * Defines callbacks for service binding, passed to bindService()
	 */
	private ServiceConnection mServiceConnection = new ServiceConnection() {

		public void onServiceConnected(ComponentName className, IBinder service) {
			MLog.i("Service bound in activity");
			RemoteServiceBinder binder = (RemoteServiceBinder) service;
			mService = binder.getService();
			mService.setHandler(MainActivity.this);

			mBound = true;

			// addStatusMsg("Service Bound");

			mHandler.sendEmptyMessage(MSG_SERVICE_BOUND);
		}

		public void onServiceDisconnected(ComponentName arg0) {
			MLog.i("Service disconnected from activity");
			mBound = false;
		}
	};

	/**
	 * @see ru.esc.btqckcmd.service.IRemoteService#onScanStarted()
	 */
	public void onScanStarted() {
		mHandler.sendEmptyMessage(MSG_SCAN_STARTED);
		addStatusMsg("Scan started");
	}

	/**
	 * @see ru.esc.btqckcmd.service.IRemoteService#onDeviceConnected()
	 */
	public void onDeviceConnected() {
		mHandler.sendEmptyMessage(MSG_CONNECTED);
		addStatusMsg("Device connected");
	}

	/**
	 * @see ru.esc.btqckcmd.service.IRemoteService#onDeviceNotFound()
	 */
	public void onDeviceNotFound() {
		mHandler.sendEmptyMessage(MSG_NOT_FOUND);
		addStatusMsg("Device not found");
	}

	/**
	 * @see ru.esc.btqckcmd.service.IRemoteService#onDeviceNotConnected()
	 */
	public void onDeviceNotConnected() {
		mHandler.sendEmptyMessage(MSG_NOT_CONNECTED);
		addStatusMsg("Device not connected");
	}

	/**
	 * @see ru.esc.btqckcmd.service.IRemoteService#onDisconnect()
	 */
	public void onDisconnect() {
		mHandler.sendEmptyMessage(MSG_DISCONNECTED);
		addStatusMsg("Disconnected");
	}

	/**
	 * Add new message to {@link #mTvStatus} text view.
	 * 
	 * @param statusMsg
	 *            - message to add.
	 */
	private void addStatusMsg(String statusMsg) {
		Message msg = new Message();
		msg.what = MSG_ADD_STATUS;
		msg.obj = statusMsg;
		mHandler.sendMessage(msg);
	}

	/**
	 * Handler used to process events in UI Thread.
	 */
	private Handler mHandler = new Handler() {
		public void handleMessage(android.os.Message msg) {
			switch (msg.what) {
			case MSG_ADD_STATUS:
				String statusMsg = (String) msg.obj;
				// mTvStatus.setText(statusMsg + "\n" + mTvStatus.getText().toString());
				break;

			case MSG_SCAN_STARTED:
				showConnectingDialog();
				break;

			case MSG_CONNECTED:
				dismissConnectingDialog();
                mBtConnect.setEnabled(true);
                mBtConnect.setText(R.string.disconnect);
				// mBtConnect.setEnabled(false);
				// mBtSendCmd.setEnabled(true);
				Toast.makeText(MainActivity.this, R.string.bt_connected,
						Toast.LENGTH_LONG).show();
				break;

			case MSG_NOT_FOUND:
				dismissConnectingDialog();
				mBtConnect.setEnabled(true);
                mBtConnect.setText(R.string.connect);
				// mBtSendCmd.setEnabled(false);
				// showChooseDeviceDialog();
				Toast.makeText(MainActivity.this, R.string.bt_not_found,
						Toast.LENGTH_LONG).show();
				break;

			case MSG_NOT_CONNECTED:
				dismissConnectingDialog();
				mBtConnect.setEnabled(true);
                mBtConnect.setText(R.string.connect);
				// mBtSendCmd.setEnabled(false);
                // showChooseDeviceDialog();
				Toast.makeText(MainActivity.this, R.string.bt_error_connecting,
						Toast.LENGTH_LONG).show();
				break;

			case MSG_DISCONNECTED:
				dismissConnectingDialog();
				mBtConnect.setEnabled(true);
                mBtConnect.setText(R.string.connect);
				// mBtSendCmd.setEnabled(false);
				Toast.makeText(MainActivity.this, R.string.bt_disconnected,
						Toast.LENGTH_LONG).show();
				break;

			case MSG_SERVICE_BOUND:
				if (mService.isConnected())
	                mBtConnect.setText(R.string.disconnect);
				else
	                mBtConnect.setText(R.string.connect);

				if (checkBluetoothState()) {
					if (mService.isConnecting()) {
						showConnectingDialog();
						return;
					}
					if (mService.isConnected())
						this.sendEmptyMessage(MSG_CONNECTED);
					else
						mService.startDeviceDiscover();
				}
				break;

			default:
				break;
			}
		};
	};

    @Override
    public void onDeviceNotFound(ArrayList<String> devicesFound) {
        onDeviceNotFound();
    }
}
